import gurobi.*;

public class Plants {
	public static void main(String[] args) throws Exception {
     double[] demands = {5, 5, 5, 5, 5, 10, 10, 15, 20, 20, 30, 30, 40, 50, 60, 60, 60, 50, 40, 30, 30, 20, 10, 5};
     int T = demands.length;

     int nBase = 10;
     double eBase = 7;
     double cBase = 2.0 / 24.0;

     int nPeak = 40;
     double ePeak = 2;
     double cPeak = 12;

     double sMax = 100;
     double gamma = 0.75;

     GRBEnv env = new GRBEnv();
		 GRBModel m = new GRBModel(env);

     // The number of running base load power plants.
		 GRBVar base = m.addVar(0.0, nBase, 0.0, GRB.INTEGER, "base");

     // How many peaking power plants are running in each hour.
		 GRBVar[] peak = new GRBVar[T];
     for (int t = 0; t < T; t++) {
         peak[t] = m.addVar(0.0, nPeak, 0.0, GRB.INTEGER, "peak_" + t);
     }

     // Energy storage state in each hour.
		 GRBVar[] s = new GRBVar[T+1];
     for (int t = 0; t < (T+1); t++) {
         s[t] = m.addVar(0.0, sMax, 0.0, GRB.CONTINUOUS, "s_" + t);
     }

     // Amount of energy taken from the storage.
		 GRBVar[] aTake = new GRBVar[T];
     for (int t = 0; t < T; t++) {
         aTake[t] = m.addVar(0.0, GRB.INFINITY, 0.0, GRB.CONTINUOUS, "aTake_" + t);
     }

     // Amount of energy put into the storage.
		 GRBVar[] aPut = new GRBVar[T];
     for (int t = 0; t < T; t++) {
         aPut[t] = m.addVar(0.0, GRB.INFINITY, 0.0, GRB.CONTINUOUS, "aPut_" + t);
     }

     // Whether we are putting or taking from the storage.
		 GRBVar[] y = new GRBVar[T];
     for (int t = 0; t < T; t++) {
         y[t] = m.addVar(0.0, 1, 0.0, GRB.BINARY, "y_" + t);
     }

		 // Integrate new variables into model.
		 m.update();

     // Constraint: satisfying demand.
     // d[t] = eBase * base + ePeak * peak[t] + gamma * aTake[t] - aPut[t]
     for (int t = 0; t < T; t++) {
         GRBLinExpr expr = new GRBLinExpr();
         expr.addTerm(eBase, base);
         expr.addTerm(ePeak, peak[t]);
         expr.addTerm(gamma, aTake[t]);
         expr.addTerm(-1.0, aPut[t]);
         m.addConstr(expr, GRB.EQUAL, demands[t], "consDemand_" + t);
     }

     // Constraint: storage stage.
     m.addConstr(s[0], GRB.EQUAL, 0, "consInitStorage_");
     for (int t = 0; t < T; t++) {
         // s[t+1] = s[t] - aTake[t] + aPut[t]
         GRBLinExpr expr = new GRBLinExpr();
         expr.addTerm(1.0, s[t]);
         expr.addTerm(-1.0, aTake[t]);
         expr.addTerm(1.0, aPut[t]);
         m.addConstr(expr, GRB.EQUAL, s[t+1], "consStorage_" + t);
     }

     // Constraint: it is not possible to take/put energy into/from storage in the same hour.
     {
         double bigM = sMax;
         for (int t = 0; t < T; t++) {
             {
                 // bigM * (1-y[t]) >= aTake[t]
                 // --->
                 // -bigM * y[t] - aTake[t] >= -bigM
                 GRBLinExpr expr = new GRBLinExpr();
                 expr.addTerm(-bigM, y[t]);
                 expr.addTerm(-1.0, aTake[t]);
                 m.addConstr(expr, GRB.GREATER_EQUAL, -bigM, "consTakeZero_" + t);
             }

             {
                 // bigM * y[t] >= aPut[t]
                 GRBLinExpr expr = new GRBLinExpr();
                 expr.addTerm(bigM, y[t]);
                 m.addConstr(expr, GRB.GREATER_EQUAL, aPut[t], "consPutZero_" + t);
             }
         }
     }

     // Objective: minimize the cost.
     // min 24 * cBase * base + sum(cPeak * peak[t] ; for t=0,...,T-1)
     {
         GRBLinExpr expr = new GRBLinExpr();
         expr.addTerm(24 * cBase, base);
         for (int t = 0; t < T; t++) {
             expr.addTerm(cPeak, peak[t]);
         }
         m.setObjective(expr, GRB.MINIMIZE);
     }

		 // Solve the model.
		 m.optimize();

		 System.out.println("Objective value: " + m.get(GRB.DoubleAttr.ObjVal));
		 System.out.println("base: " + base.get(GRB.DoubleAttr.X));
     for (int t = 0; t < T; t++) {
         System.out.println("peak[" + t + "]: "+ peak[t].get(GRB.DoubleAttr.X));
     }
     for (int t = 0; t < (T+1); t++) {
         System.out.println("s[" + t + "]: "+ s[t].get(GRB.DoubleAttr.X));
     }

     for (int t = 0; t < T; t++) {
         System.out.println("aTake[" + t + "]: "+ aTake[t].get(GRB.DoubleAttr.X) + ", aPut[" + t + "]: "+ aPut[t].get(GRB.DoubleAttr.X));
     }
	}
}
